#!/usr/bin/env python3

"""Grupo de rotas para balaios."""
from typing import Optional, List

from fastapi import APIRouter, Form, Header

from baobaxia.core.baobaxia_api import bbx
from baobaxia.saberes import Balaio

router = APIRouter(tags=["Balaios"])


@router.get(
    "/balaio/default", response_model=Balaio, summary="Retornar o balaio padrão"
)
async def get_default_balaio(token: Optional[str] = Header(None)):
    """Retornar o balaio padrão."""
    return bbx.baobaxia.get_default_balaio(token=token)


@router.get("/balaio", response_model=List[Balaio], summary="Listar balaios")
async def list_balaios(token: Optional[str] = Header(None)):
    """Listar balaios."""
    return bbx.baobaxia.list_balaios(token=token)


@router.post("/balaio", response_model=Balaio, summary="Criar um novo balaio")
async def post_balaio(
    name: str = Form(...), default_mucua: str = Form(...), token: str = Header(...)
):
    """Criar um novo balaio."""
    return bbx.baobaxia.put_balaio(name=name, default_mucua=default_mucua, token=token)


@router.get(
    "/balaio/{balaio_slug_smid}", response_model=Balaio, summary="Retornar um balaio"
)
async def get_balaio(balaio_slug_smid: str, token: Optional[str] = Header(None)):
    """Retornar um balaio."""
    # TODO: self => bbx.extract...
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    return bbx.baobaxia.get_balaio(balaio_smid, token=token)


@router.put(
    "/balaio/{balaio_slug_smid}", response_model=Balaio, summary="Atualizar um balaio"
)
async def put_balaio(
    balaio_slug_smid: str,
    name: str = Form(...),
    default_mucua: str = Form(...),
    token: str = Header(...),
):
    """Atualizar um balaio."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    return bbx.baobaxia.put_balaio(
        smid=balaio_smid, name=name, default_mucua=default_mucua, token=token
    )


@router.delete("/balaio/{balaio_slug_smid}", summary="Deletar um balaio")
async def del_balaio(balaio_slug_smid: str, token: str = Header(...)):
    """Deletar um balaio."""
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    bbx.baobaxia.del_balaio(balaio_smid, token=token)
    return {"detail": "success"}
