#!/usr/bin/env python3

"""Grupo de rotas para autenticação."""
from fastapi import APIRouter, Header
from typing import Optional
from baobaxia.core.baobaxia_api import bbx
from baobaxia.saberes import Pedido
from pathlib import Path


router = APIRouter(tags=["Pedidos"])


# PEDIDOS
@router.post(
    "/{balaio_slug_smid}/{mucua_slug_smid}/pedido/{slug_smid:str}",
    response_model=dict,
    summary="Criar um pedido de copia de um saber",
)
async def post_pedido(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    slug_smid: str,
    ttl: int,
    token: str = Header(...),
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    smid = bbx.extract_smid(slug_smid)
    # mocambola = bbx.baobaxia.get_session(token).mocambola
    midia = bbx.baobaxia.get_midia(balaio_smid, mucua_smid, smid, token)
    pedido = Pedido(
        name="Pedido  " + midia.titulo,
        path=Path("pedidos/"),
        midia_balaio_smid=midia.balaio_smid,
        midia_mucua_smid=midia.mucua_smid,
        midia_smid=midia.smid,
        ttl=3,
    )
    bbx.baobaxia.put_pedido(balaio_smid, mucua_smid, pedido, token)
    midia.is_requested = True
    bbx.baobaxia.put_midia(balaio_smid, mucua_smid, midia, token)
    return {"detail": "success"}


@router.get(
    "/{balaio_slug_smid}/{mucua_slug_smid}/pedidos", summary="Listar os Pedidos"
)
async def list_pedidos(
    balaio_slug_smid: str,
    mucua_slug_smid: str,
    token: Optional[str] = Header(None),
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    return bbx.baobaxia.list_pedidos(balaio_smid, mucua_smid, token=token)


@router.post(
    "/{balaio_slug_smid}/{mucua_slug_smid}/processar_pedidos",
    summary="Processar pedidos",
)
async def processar_pedidos(
    balaio_slug_smid: str, mucua_slug_smid: str, token: str = Header(...)
):
    balaio_smid = bbx.extract_smid(balaio_slug_smid)
    mucua_smid = bbx.extract_smid(mucua_slug_smid)
    mocambola = bbx.baobaxia.get_session(token).mocambola
    pedidos = bbx.baobaxia.list_pedidos(balaio_smid, mucua_smid, token)
    rotas = bbx.baobaxia.list_rotas(balaio_smid, mucua_smid, token)
    # Coletando tambem os pedidos das mucuas nas rotas
    for rota in rotas:
        for mucua_smid in rota.mucuas_smid:
            pedidos.extend(bbx.baobaxia.list_pedidos(balaio_smid, mucua_smid, token))
    bbx.baobaxia.get_copy(
        pedidos=pedidos,
        token=token,
        mocambola=mocambola,
        config=bbx.baobaxia.config,
    )
    return {"detail": "success"}
