#!/usr/bin/env python3
from typing import Optional, List
from fastapi import Header, APIRouter

from baobaxia.root import Baobaxia


class BaobaxiaAPI:
    def __init__(self):
        """Inicializa Baobaxia wrapper."""
        self.baobaxia = Baobaxia()

    def extract_smid(self, slug_smid: str):
        # TODO: Usar split fica mais simples de entender
        return slug_smid[-self.baobaxia.config.smid_len :]

    def add_saberes_api(self, model: type, router: APIRouter, **kwargs):
        if "field_name" in kwargs and kwargs["field_name"] is not None:
            field_name = kwargs["field_name"]
        else:
            field_name = model.__name__.lower()
        if "list_field_name" in kwargs and kwargs["list_field_name"] is not None:
            list_field_name = kwargs["list_field_name"]
        else:
            list_field_name = field_name + "s"
        if "tags" in kwargs:
            tags = kwargs["tags"]
        else:
            tags = [""]

        if "url_path" in kwargs:
            url_path = "/{balaio_slug_smid}/{mucua_slug_smid}/" + kwargs["url_path"]
        else:
            url_path = "/{balaio_slug_smid}/{mucua_slug_smid}/" + field_name

        if "list_summary" in kwargs:
            summary = kwargs["list_summary"]
        else:
            summary = "Listar " + list_field_name
        if "list_url" in kwargs:
            list_url = kwargs["list_url"]
        else:
            list_url = url_path
        if "list_method" in kwargs and callable(kwargs["list_method"]):
            router.add_api_route(
                list_url,
                kwargs["list_method"],
                response_model=List[model],
                methods=["GET"],
                summary=summary,
                tags=tags,
            )
        elif "skip_list_method" not in kwargs:

            def list_rest_template(
                *,
                balaio_slug_smid: Optional[str],
                mucua_slug_smid: Optional[str],
                token: Optional[str] = Header(None)
            ):
                balaio_smid = self.extract_smid(balaio_slug_smid)
                mucua_smid = self.extract_smid(mucua_slug_smid)
                return getattr(self.baobaxia, "list_" + list_field_name)(
                    balaio_smid, mucua_smid, token
                )

            router.add_api_route(
                list_url,
                list_rest_template,
                response_model=List[model],
                methods=["GET"],
                summary=summary,
                tags=tags,
            )

        if "get_summary" in kwargs:
            summary = kwargs["get_summary"]
        else:
            summary = "Retornar " + field_name
        if "get_url" in kwargs:
            get_url = kwargs["get_url"]
        else:
            get_url = url_path + "/{slug_smid:str}"
        if "get_method" in kwargs and callable(kwargs["get_method"]):
            router.add_api_route(
                get_url,
                kwargs["get_method"],
                response_model=model,
                methods=["GET"],
                summary=summary,
                tags=tags,
            )
        elif "skip_get_method" not in kwargs:

            def get_rest_template(
                *,
                balaio_slug_smid: str,
                mucua_slug_smid: str,
                slug_smid: str,
                token: Optional[str] = Header(None)
            ):
                balaio_smid = self.extract_smid(balaio_slug_smid)
                mucua_smid = self.extract_smid(mucua_slug_smid)
                smid = self.extract_smid(slug_smid)
                return getattr(self.baobaxia, "get_" + field_name)(
                    balaio_smid, mucua_smid, smid, token
                )

            router.add_api_route(
                get_url,
                get_rest_template,
                response_model=model,
                methods=["GET"],
                summary=summary,
                tags=tags,
            )
        # INICIO
        if "post_summary" in kwargs:
            summary = kwargs["post_summary"]
        else:
            summary = "Criar " + field_name
        if "post_url" in kwargs:
            post_url = kwargs["post_url"]
        else:
            post_url = url_path
        if "post_method" in kwargs and callable(kwargs["post_method"]):
            router.add_api_route(
                post_url,
                kwargs["post_method"],
                response_model=model,
                methods=["POST"],
                summary=summary,
            )
        elif "skip_put_method" not in kwargs:

            def post_rest_template(
                *,
                balaio_slug_smid: str,
                mucua_slug_smid: str,
                data: model,
                token: str = Header(...)
            ):
                balaio_smid = self.extract_smid(balaio_slug_smid)
                mucua_smid = self.extract_smid(mucua_slug_smid)
                return getattr(self.baobaxia, "put_" + field_name)(
                    balaio_smid, mucua_smid, data, token
                )

            router.add_api_route(
                post_url,
                post_rest_template,
                response_model=model,
                methods=["POST"],
                summary=summary,
                tags=tags,
            )
        # TERMINA

        if "put_summary" in kwargs:
            summary = kwargs["put_summary"]
        else:
            summary = "Atualizar " + field_name
        if "put_url" in kwargs:
            put_url = kwargs["put_url"]
        else:
            put_url = url_path
        if "put_method" in kwargs and callable(kwargs["put_method"]):
            router.add_api_route(
                put_url,
                kwargs["put_method"],
                response_model=model,
                methods=["PUT"],
                summary=summary,
                tags=tags,
            )
        elif "skip_put_method" not in kwargs:

            def put_rest_template(
                *,
                balaio_slug_smid: str,
                mucua_slug_smid: str,
                slug_smid: str,
                data: model,
                token: str = Header(...)
            ):
                balaio_smid = self.extract_smid(balaio_slug_smid)
                mucua_smid = self.extract_smid(mucua_slug_smid)
                smid = self.extract_smid(slug_smid)
                assert smid == data.smid
                return getattr(self.baobaxia, "put_" + field_name)(
                    balaio_smid, mucua_smid, smid, data, token
                )

            router.add_api_route(
                put_url,
                put_rest_template,
                response_model=model,
                methods=["PUT"],
                summary=summary,
                tags=tags,
            )

        if "del_summary" in kwargs:
            summary = kwargs["del_summary"]
        else:
            summary = "Deletar " + field_name
        if "del_url" in kwargs:
            del_url = kwargs["del_url"]
        else:
            del_url = url_path + "/{slug_smid:str}"
        if "del_method" in kwargs and callable(kwargs["del_method"]):
            router.add_api_route(
                del_url,
                kwargs["del_method"],
                methods=["DELETE"],
                summary=summary,
                tags=tags,
            )
        elif "skip_del_method" not in kwargs:

            def del_rest_template(
                *,
                balaio_slug_smid: str,
                mucua_slug_smid: str,
                slug_smid: str,
                token: str = Header(...)
            ):
                balaio_smid = self.extract_smid(balaio_slug_smid)
                mucua_smid = self.extract_smid(mucua_slug_smid)
                smid = self.extract_smid(slug_smid)
                getattr(self.baobaxia, "del_" + field_name)(
                    balaio_smid, mucua_smid, smid, token
                )
                return {"detail": "success"}

            router.add_api_route(
                del_url,
                del_rest_template,
                methods=["DELETE"],
                summary=summary,
                tags=tags,
            )


bbx = BaobaxiaAPI()
