#!/usr/bin/env python3

from typing import Optional, List, Dict
from pathlib import Path
from enum import Enum
from baobaxia.saberes import Saber


class MidiaTipo(str, Enum):
    video = "video"
    audio = "audio"
    imagem = "imagem"
    arquivo = "arquivo"


class MidiaStatus(str, Enum):
    draft = "draft"
    published = "published"


class ArtigoStatus(str, Enum):
    draft = "draft"
    published = "published"


class Midia(Saber):
    titulo: str
    descricao: Optional[str] = None
    autor: Optional[str] = None
    mocambo: Optional[str] = None
    tipo: Optional[MidiaTipo] = None
    status: MidiaStatus = MidiaStatus.draft
    tags: List[str] = []


class Artigo(Saber):
    titulo: str
    path: Path = Path("blog")
    resumo: Optional[str] = None
    texto: str
    status: ArtigoStatus = ArtigoStatus.draft
    tags: List[str] = []


class Selecao(Saber):
    titulo: str
    path: Path = Path("selecao")
    saberes: List[Dict[str, str]] = []
    tags: List[str] = []
