from pathlib import Path

from .test_env import BaseBaobaxiaTest, DataTestModel

from baobaxia.root import Baobaxia


class TestBaobaxia(BaseBaobaxiaTest):

    def token(self, baobaxia):
        return baobaxia.authenticate(
            self.mocambola_username,
            password=self.mocambola_password
        ).token

    def test_list_balaios(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        balaios_list = baobaxia.list_balaios(token)
        assert len(balaios_list) > 0

    def test_get_balaio(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        balaio = baobaxia.get_balaio(self.balaio_smid, token)
        assert balaio.smid == self.balaio_smid
        assert balaio.name == self.balaio_name

    def test_put_balaio(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        balaio = baobaxia.get_balaio(self.balaio_smid, token)
        old_name = balaio.name
        new_name = "Nome alterado"
        balaio = baobaxia.put_balaio(
            smid = self.balaio_smid,
            name = new_name,
            default_mucua = self.mucua_smid,
            token = token)
        assert balaio.name == new_name
        balaio = baobaxia.put_balaio(
            smid = self.balaio_smid,
            name = old_name,
            default_mucua = self.mucua_smid,
            token = token)
        assert balaio.name == old_name
        new_name = "Novo balaio"
        balaio = baobaxia.put_balaio(
            name = new_name,
            default_mucua = self.mucua_smid,
            token = token)
        assert balaio.smid is not None
        assert balaio.name == new_name

    def test_add_mocambola_balaio(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        balaio = baobaxia.get_balaio(self.balaio_smid, token)
        novo_balaio = baobaxia.put_balaio(
            name = "Mais um balaio",
            default_mucua = self.mucua_smid,
            token = token)
        baobaxia.add_mocambola_balaio(
            balaio_smid = balaio.smid,
            mucua_smid = self.mucua_smid,
            novo_balaio_smid = novo_balaio.smid,
            username = self.mocambola_username,
            token = token)
        mocambola_em_novo_balaio = baobaxia.get_mocambola(
            balaio_smid = novo_balaio.smid,
            mucua_smid = self.mucua_smid,
            username = self.mocambola_username,
            token = token)
        assert mocambola_em_novo_balaio.balaio_smid == novo_balaio.smid
        assert mocambola_em_novo_balaio.username == self.mocambola_username
        
    def test_del_balaio(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        balaio_name = "Balaio para deletar"
        balaio = baobaxia.put_balaio(
            name = balaio_name,
            token = token)
        balaio_smid = balaio.smid
        baobaxia.del_balaio(
            balaio_smid,
            token)
        try:
            baobaxia.get_balaio(balaio_smid)
        except:
            pass
        else:
            assert False

    def test_list_mucuas(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        mucuas = baobaxia.list_mucuas(
            balaio_smid = self.balaio_smid,
            token = token)
        assert len(mucuas) > 0

    def test_get_mucua(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        mucua = baobaxia.get_mucua(
            balaio_smid = self.balaio_smid,
            mucua_smid = self.mucua_smid,
            token = token)
        mucua.smid = self.mucua_smid
        mucua.name = self.mucua_name


    def test_put_mucua(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        mucua = baobaxia.get_mucua(
            balaio_smid = self.balaio_smid,
            mucua_smid = self.mucua_smid,
            token = token)
        old_name = mucua.name
        new_name = "Nome alterado"
        mucua = baobaxia.put_mucua(
            balaio_smid = self.balaio_smid,
            mucua_smid = self.mucua_smid,
            name = new_name,
            token = token)
        assert mucua.name == new_name
        mucua = baobaxia.put_mucua(
            balaio_smid = self.balaio_smid,
            mucua_smid = self.mucua_smid,
            name = old_name,
            token = token)
        assert mucua.name == old_name
        new_name = "Nova mucua"
        mucua = baobaxia.put_mucua(
            balaio_smid = self.balaio_smid,
            name = new_name,
            token = token)
        assert mucua.smid is not None
        assert mucua.name == new_name

    def test_del_mucua(self):
        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        mucua_name = "Mucua para deletar"
        mucua = baobaxia.put_mucua(
            balaio_smid = self.balaio_smid,
            name = mucua_name,
            token = token)
        mucua_smid = mucua.smid
        baobaxia.del_mucua(
            balaio_smid = self.balaio_smid,
            mucua_smid = mucua_smid,
            token = token)
        try:
            baobaxia.get_mucua(
                balaio_smid = self.balaio_smid,
                mucua_smid = mucua_smid,
                token = token)
        except:
            pass
        else:
            assert False

    def test_discover_saberes(self):
        test_dir = self.config.data_path / \
            self.balaio_slug / self.mucua_slug / 'test'
        if not test_dir.exists():
            test_dir.mkdir()

        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        baobaxia.discover_saberes(
            model = DataTestModel,
            patterns = ['test/*'])
        assert hasattr(baobaxia, 'list_datatestmodels')
        assert hasattr(baobaxia, 'get_datatestmodel')
        assert hasattr(baobaxia, 'put_datatestmodel')
        #assert hasattr(baobaxia, 'post_datatestmodel')
        assert hasattr(baobaxia, 'del_datatestmodel')

        test_name = 'meu_teste'
        test_path = Path('test/')
        test_message = 'Isto é um teste'
        test_saber = baobaxia.put_datatestmodel(
            self.balaio_slug, self.mucua_slug,
            DataTestModel(name=test_name, path=test_path,message = test_message),
            token)
        test_smid = test_saber.smid

        baobaxia = Baobaxia(self.config)
        token = self.token(baobaxia)
        baobaxia.discover_saberes(
            model = DataTestModel,
            patterns = ['test/*/'],
            field_name = 'test',
            list_field_name = 'tests_collection')

        tests = baobaxia.list_tests_collection(
            self.balaio_slug, self.mucua_slug, token)
        assert len(tests) > 0
        test_saber = baobaxia.get_test(
            self.balaio_slug, self.mucua_slug,
            smid = test_saber.smid,
            token = token)
        assert test_smid == test_saber.smid
        assert test_name == test_saber.name
        assert test_message == test_saber.message

        baobaxia.del_test(
            self.balaio_slug, self.mucua_slug,
            smid = test_saber.smid,
            token = token)

